@@ -49,7 +49,8 @@ def iterate_tensor(t):
 
 # full attention for calculating auxiliary reconstruction loss
 def full_attn(q, k, v):
-    dots = torch.einsum('bhid,bhjd->bhij', q, k)
+    *_, dim = q.shape
+    dots = torch.einsum('bhid,bhjd->bhij', q, k) * (dim ** -0.5)
     attn = dots.softmax(dim=-1)
     return torch.einsum('bhij,bhjd->bhid', attn, v)
 
@@ -216,7 +217,7 @@ class SelfAttention(nn.Module):
         aux_loss = torch.zeros(1, requires_grad = True, **to(q))
 
         if self.seq_len < t or not calc_memory:
-            return logits, Memory(mem = new_mem, compressed_mem = new_cmem), aux_loss
+            return logits, Memory(new_mem, new_cmem), aux_loss
 
         # calculate memory and compressed memory
 
@@ -248,17 +249,21 @@ class SelfAttention(nn.Module):
                 )
 
 
-        return logits, Memory(mem = new_mem, compressed_mem = new_cmem), aux_loss
+        return logits, Memory(new_mem, new_cmem), aux_loss
 
 # transformer
 
 class CompressiveTransformer(nn.Module):
-    def __init__(self, num_tokens, dim, seq_len, depth, memory_layers = None, mem_len = None, cmem_len = None, cmem_ratio = 4, heads = 8, gru_gated_residual = True, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., reconstruction_loss_weight = 1., one_kv_head = False):
+    def __init__(self, num_tokens, dim, seq_len, depth, memory_layers = None, mem_len = None, cmem_len = None, cmem_ratio = 4, heads = 8, gru_gated_residual = True, attn_dropout = 0., ff_glu = False, ff_dropout = 0., attn_layer_dropout = 0., reconstruction_loss_weight = 1., one_kv_head = False):
         super().__init__()
         mem_len = default(mem_len, seq_len)
         cmem_len = default(cmem_len, mem_len // cmem_ratio)
         memory_layers = default(memory_layers, list(range(1, depth + 1)))
 
+        assert mem_len >= seq_len, 'length of memory should be at least the sequence length'
+        assert cmem_len >= (mem_len // cmem_ratio), f'length of compressed memory should be at least the memory length divided by the compression ratio {int(mem_len // cmem_ratio)}'
+        assert all([layer > 0 and layer <= depth for layer in memory_layers]), 'one of the indicated memory layers is invalid'
+
         self.seq_len = seq_len
 
         self.depth = depth
@@ -273,13 +278,14 @@ class CompressiveTransformer(nn.Module):
         wrapper = partial(GRUGating, dim) if gru_gated_residual else Residual
 
         self.attn_layers = nn.ModuleList([wrapper(PreNorm(dim, SelfAttention(dim, seq_len, mem_len, cmem_len, cmem_ratio, heads, dropout = attn_layer_dropout, attn_dropout = attn_dropout, one_kv_head = one_kv_head))) for _ in range(depth)])
-        self.ff_layers = nn.ModuleList([wrapper(PreNorm(dim, FeedForward(dim, dropout = ff_dropout))) for _ in range(depth)])
+        self.ff_layers = nn.ModuleList([wrapper(PreNorm(dim, FeedForward(dim, dropout = ff_dropout, glu = ff_glu))) for _ in range(depth)])
 
         self.reconstruction_loss_weight = reconstruction_loss_weight
 
     def forward(self, x, memories = None, mask = None):
         x = self.token_emb(x)
         b, t, d = x.shape
+        assert t <= self.seq_len, f'input contains a sequence length {t} that is greater than the designated maximum sequence length {self.seq_len}'
 
         mem = cmem = None
         if memories is not None:
